#include <BLEDevice.h>
#include <BLEServer.h>
#include <BLEUtils.h>
#include <BLE2902.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

BLEServer *pServer = NULL;
BLECharacteristic *paCharacteristic = NULL;
BLECharacteristic *psiCharacteristic = NULL;
BLECharacteristic *kgfCharacteristic = NULL;

#define SERVICE_UUID "f1a2b3c4-d5e6-7890-abc1-234567890def"
#define PA_CHARACTERISTIC_UUID "a3b4c5d6-e7f8-9012-abcd-34567890fedc"
#define PSI_CHARACTERISTIC_UUID "b4c5d6e7-f8a9-0123-bcde-456789012345"
#define KGF_CHARACTERISTIC_UUID "c5d6e7f8-a9b0-1234-cdef-567890123456"

#define UnitButton_PIN 7
#define BLUELED_PIN 12

int BLEbuttonState = 0;
int BLElastButtonState = 0;
int BLEunitIndex = 0;
bool deviceConnected = false;
bool advertisingStarted = false;
bool BLEledState = LOW;

unsigned long BLEpreviousMillis = 0;
const long BLEinterval = 500;

LiquidCrystal_I2C lcd(0x27, 20, 4);

class MyServerCallbacks : public BLEServerCallbacks {
  void onConnect(BLEServer* pServer) {
    deviceConnected = true;
    digitalWrite(BLUELED_PIN, HIGH);
    lcd.setCursor(2, 2);
    lcd.print("Device Connected  ");
    delay(3000);
    digitalWrite(BLUELED_PIN, LOW);
    clearRow(1);
    clearRow(2);
    clearRow(3);
  }

  void onDisconnect(BLEServer* pServer) {
    deviceConnected = false;
    advertisingStarted = false;
    digitalWrite(BLUELED_PIN, LOW);
    clearRow(1);
    clearRow(2);
    clearRow(3);
  }
};

void clearRow(int row) {
  lcd.setCursor(0, row);
  lcd.print("                    ");
}

void BLEsetup() {
  lcd.init();
  lcd.backlight();
  Serial.begin(115200);

  pinMode(UnitButton_PIN, INPUT_PULLUP);
  pinMode(BLUELED_PIN, OUTPUT);

  digitalWrite(BLUELED_PIN, LOW);

  BLEDevice::init("ESP32 Algometer AQ");
  pServer = BLEDevice::createServer();
  pServer->setCallbacks(new MyServerCallbacks());

  BLEService *pService = pServer->createService(SERVICE_UUID);

  paCharacteristic = pService->createCharacteristic(PA_CHARACTERISTIC_UUID, BLECharacteristic::PROPERTY_NOTIFY);
  psiCharacteristic = pService->createCharacteristic(PSI_CHARACTERISTIC_UUID, BLECharacteristic::PROPERTY_NOTIFY);
  kgfCharacteristic = pService->createCharacteristic(KGF_CHARACTERISTIC_UUID, BLECharacteristic::PROPERTY_NOTIFY);

  BLE2902 *pBLE2902 = new BLE2902();
  pBLE2902->setNotifications(true);
  paCharacteristic->addDescriptor(pBLE2902);
  psiCharacteristic->addDescriptor(pBLE2902);
  kgfCharacteristic->addDescriptor(pBLE2902);

  pService->start();
}

void BLEloop() {
  BLEbuttonState = digitalRead(UnitButton_PIN);
  if (BLEbuttonState == LOW && BLElastButtonState == HIGH) {
    BLEunitIndex = (BLEunitIndex + 1) % 2;

    if (BLEunitIndex == 1) {
      startBLEAdvertising();
    } else if (BLEunitIndex == 0) {
      clearRow(2);
      digitalWrite(BLUELED_PIN, LOW);
      stopBLEAdvertising();
    }
    delay(200);
  }

  BLElastButtonState = BLEbuttonState;

  if (deviceConnected) {
    digitalWrite(BLUELED_PIN, HIGH);
  } else if (advertisingStarted) {
    unsigned long BLEcurrentMillis = millis();
    if (BLEcurrentMillis - BLEpreviousMillis >= BLEinterval) {
      BLEpreviousMillis = BLEcurrentMillis;
      BLEledState = !BLEledState;
      digitalWrite(BLUELED_PIN, BLEledState);
    }
  }
}

void startBLEAdvertising() {
  BLEAdvertising *pAdvertising = BLEDevice::getAdvertising();
  pAdvertising->addServiceUUID(SERVICE_UUID);
  pAdvertising->setScanResponse(false);
  pAdvertising->setMinPreferred(0x0);
  BLEDevice::startAdvertising();
  advertisingStarted = true;
  lcd.setCursor(2, 2);
  lcd.print("Device Connecting");
}

void stopBLEAdvertising() {
  BLEDevice::stopAdvertising();
  advertisingStarted = false;
  deviceConnected = false;
  clearRow(1);
  clearRow(2);
  clearRow(3);
}
